#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from psas_packet import messages

HEADER = messages.Head()
PSAS = {cls.fourcc: cls for cls in messages.PSAS_MESSAGES}


def _is_string_like(obj):
    """
    Check whether obj behaves like a string.
    """
    try:
        obj + ''
    except (TypeError, ValueError):
        return False
    return True


class BlockSize(Exception):

    def __init__(self):
        msg = "Block to decode is too small"
        Exception.__init__(self, msg)


def decode_block(block):
    """Decode a single message from a block of bytes.

    :param bytes block: bytes to try and decode
    :returns: number of bytes read, dictionary with unpacked values

    Raises exeptions if block is not readable
    """

    # HEADER:

    # make sure we have enough bytes to deocde a header, then do it
    if len(block) < HEADER.size:
        raise(BlockSize)
        return
    info = HEADER.decode(block[:HEADER.size])

    # DATA:

    # figure out what type it is based on FOURCC, and get that message class
    message_cls = PSAS.get(info['fourcc'], None)

    # Yay! We know about this type, lets unpack it
    if message_cls is not None:
        if len(block) < HEADER.size+message_cls.size:
            raise(BlockSize)
            return
        unpacked = message_cls.decode(block[HEADER.size:HEADER.size+message_cls.size])
        return HEADER.size+message_cls.size, {info['fourcc']: dict({'timestamp': info['timestamp']}, **unpacked)}


class BinFile(object):
    """Read from a binary log file

    :param fname: A filename or file-like object
    :returns: BinFile object

    """

    def __init__(self, fname):

        # Try and see if the passed in file is filename (string) or an object that might act like a file
        if _is_string_like(fname):
            self.fh = open(fname, 'rb')
        else:
            self.fh = fname

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        self.fh.close()

    def read(self):
        """Read the file and return data inside it
        """

        # Read a chunk of file
        buff = self.fh.read(1 << 20)  # 1 MB

        # As long as we have something to look at
        while buff != b'':
            try:
                bytes_read, data = decode_block(buff)
                buff = buff[bytes_read:]
                # Our buffer got too small, read some more from the file
                # All messages start with a fourcc, so if there isn't at leas 4 bytes we've lost.
                if len(buff) < 4:
                    buff += self.fh.read(1 << 20)  # 1 MB
                yield data
            except (BlockSize):
                b = self.fh.read(1 << 20)  # 1 MB
                # Check that we didn't actually hit the end of the file
                if b == b'':
                    break
                buff += b



# -*- coding: utf-8 -*-
"""Network stack for exchanging packets with messages.
"""
import socket

TELEMETRY_IP = "127.0.0.1"
TELEMETRY_PORT = 35001


class SendUDP(object):
    """UDP socket sender context

    :param string addr: IP Address to send to
    :param int send_port: Port number to send to
    :param int from_port: Port number to send from (default=0)
    :returns: SendUDP instance

    Example use with a context manager::

        with SendUDP('127.0.0.1', 4321) as udp:
            udp.send_message(msgtype, data)

    """

    def __init__(self, addr, send_port, from_port=0):
        self.ip_address = addr
        self.send_port = send_port
        self.from_port = from_port

        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.socket.bind(('', self.from_port))
        self.socket.connect((self.ip_address, self.send_port))

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        self.socket.close()

    def close(self):
        """Release the socket
        """
        self.socket.close()

    def send_message(self, msgtype, data):
        """Send message over socket. Does the packing for you.

        :param Message msgtype: Message class to use for packing, see: psas_packet.messages
        :param dict data: Data to get packed and sent

        """
        try:
            self.socket.send(msgtype.encode(data))
        except:
            pass

    def send_raw(self, raw):
        """Send raw bytes using this connection

        :param raw bytes: bytes to send

        """
        try:
            self.socket.send(raw)
        except:
            pass


class ListenUDP(object):
    """UDP socket listener context
    """

    def __init__(self, listen_port, bind=''):
        self.bind_addr = bind
        self.listen_port = listen_port

        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.socket.bind((self.bind_addr, self.listen_port))
        self.socket.settimeout(0.01)

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        self.close()

    def close(self):
        """Release the socket
        """
        self.socket.close()

    def listen(self):
        """Listen for messages on the socket
        """
        data = None
        try:
            data, addr = self.socket.recvfrom(1400)
        except socket.timeout:
            pass

        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


setup(
    name='psas_packet',
    version='0.1.13',
    description='serializer for PSAS data standards',
    long_description=open('README.rst').read(),
    author='Nathan Bergey',
    author_email='nathan.bergey@gmail.com',
    url='http://psas-packet-serializer.readthedocs.org',
    packages=['psas_packet'],
    package_dir={'psas_packet': 'psas_packet'},
    include_package_data=True,
    install_requires=[],
    scripts=[
        'scripts/gen-psas-types',
        'scripts/log2csv',
        'scripts/slicelog',
        'scripts/replaylog',
    ],
    license=open('LICENSE').read(),
    zip_safe=False,
    classifiers=[
        'Natural Language :: English',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
    ],
    test_suite='tests',
)
